# Anthology Django

## Requirements

* Python 3 (~ 3.8)
* Docker

## Setup

### Docker deployment

Run server docker :
* `docker-compose up -d`

Setup the database:
* `docker-compose exec web python manage.py migrate`

Import AP data into the database:
* `docker-compose exec web python manage.py import_ap`
* Possible options include:
  * `docker-compose exec web python manage.py import_ap --save`: saves JSON locally on disk (see next)
  * `docker-compose exec web python manage.py import_ap --local`: loads JSON from disk (you MUST run the command above first)

Then you can access the server at http://localhost:8000


### Development deployment

#### Install Database (Postgres) and db management
* Install PostgreSQL + PostGIS + connector : `sudo apt install libpq-dev postgresql postgresql-contrib postgis`
* Connect to database :
```
    sudo su postgres (login as the user "postgres", if normal user havent the right to access the database)
    psql
```

* Create database ([doc](https://www.digitalocean.com/community/tutorials/how-to-use-postgresql-with-your-django-application-on-ubuntu-14-04)) and configure for django:
 ```
    CREATE DATABASE anthology;
    CREATE USER anthology_django WITH PASSWORD 'password';
    ALTER ROLE anthology_django SET client_encoding TO 'utf8';
    ALTER ROLE anthology_django SET default_transaction_isolation TO 'read committed';
    ALTER ROLE anthology_django SET timezone TO 'UTC-4';
    ALTER USER anthology_django CREATEDB;
    GRANT ALL PRIVILEGES ON DATABASE anthology TO anthology_django;
    CREATE EXTENSION postgis;
```

* (Optional) Drop database
When updating the schema, before running `makemigration` having no data yet, you want to drop existing db to 
recreate it.

 ```
    psql
    DROP DATABASE anthology;
    CREATE DATABASE anthology;
    GRANT ALL PRIVILEGES ON DATABASE anthology TO anthology_django;
```


* ONLY FOR TESTING ENV:
```
ALTER ROLE anthology_django SUPERUSER;
```

#### Setup Python environment

* In Windows: `py -3.8 -m venv venv`
* In Linux/macOS: `python3.8 -m venv venv`

> This command MUST be run with Python 3.8.

Activate the virtual environment:
* In Windows (Powershell): `.\venv\Scripts\activate.ps1` (`activate.bat` in the command line)
* Linux / macOS: `source venv/bin/activate`

Install the dependencies:
* `pip install -r requirements.txt`

#### Running server

 * Activate the virtual environment/
 * (first time) `python manage.py migrate`
 * (first time) `python manage.py import_ap`
 * `python manage.py runserver 0.0.0.0:8000`

#### Updating settings.py to run in localhost or inside Docker

Change the database in `anthology/settings.py` :

##### Docker version

> Do not change anything. The default 'db' host matches the PostgreSQL container.

##### Localhost version
In this case, you have to have a running instance of Postgres (see section above)

```python
DATABASES = {
    'default': {
        'ENGINE': 'django.contrib.gis.db.backends.postgis',
        'NAME': 'anthology',
        'USER': 'anthology_django',
        'PASSWORD': 'password',
        'HOST': 'localhost',
        'PORT': 5432,
    }
}
```
 
#### Updating the model

* make change in the `anthology/api/models` folder
* run `python manage.py makemigrations`
* run `python manage.py migrate`

##### To test it in docker :

* set settings.py to use docker
* (in the case of no data has been put in db yet)
    * stop the docker-compose : `docker-compose stop`
    * delete old containers : `docker-compose rm`
* `docker-compose up -d --build`
* `docker-compose exec web python manage.py migrate`  

