from meleager.models import Author, Name

from ..constants import API_URL, LANG_API2CODE
from ..helpers import to_date
from .cities import CityFromApi
from .data_manager import DataManager
from .users import UserFromApi


class AuthorFromApi:
    _authors = dict()

    def __new__(cls, id_author):
        # Check if author exists etc
        if id_author not in cls._authors:
            author_obj = cls._create_author(id_author)
            cls._authors[id_author] = author_obj
        else:
            author_obj = cls._authors[id_author]

        return author_obj

    @classmethod
    def _create_author(cls, id_author):
        json_data = DataManager.get(f"{API_URL}/v1/authors/{id_author}")
        user_obj = UserFromApi(json_data["id_user"]["id_user"])

        # Create names
        names = [
            Name.objects.create(
                name=version["name"],
                language_id=LANG_API2CODE[version["id_language"]],
                creator=user_obj,
                last_editor=user_obj,
                created_at=to_date(version["createdAt"]),
                updated_at=to_date(version["updatedAt"]),
            )
            for version in json_data["versions"]
        ]

        city_born = CityFromApi(json_data.get("city_born"))
        city_died = CityFromApi(json_data.get("city_died"))

        author_obj = Author.objects.create(
            city_born=city_born,
            city_died=city_died,
        )

        author_obj.names.add(*names)

        return author_obj
