from django.db import models

from .mixins import DescriptableResourceMixin, EditableResourceMixin


class CommentType(models.TextChoices):
    USER_NOTE = ("user_note", "A note by an user")


class Comment(EditableResourceMixin, DescriptableResourceMixin, models.Model):
    images = models.ManyToManyField(
        "meleager.Medium",
        related_name="comments",
        help_text="The images associated to this resource.",
        blank=True,
    )
    comment_type = models.CharField(max_length=20, choices=CommentType.choices, default=CommentType.USER_NOTE, null=False, db_index=True)
    comment_title = models.TextField()

    def __str__(self):
        if self.descriptions.count():
            return f"Comment {self.descriptions.first().description}"
        else:
            return f"Comment ({self.comment_title})"
