from django.db import models

from .mixins import DescriptableResourceMixin, EditableResourceMixin


class Edition(EditableResourceMixin, DescriptableResourceMixin, models.Model):
    class EditionType(models.IntegerChoices):
        MANUSCRIPT = 0
        TRANSLATION = 1
        COMMENTARY = 2
        SCHOLIA = 3
        ALIGNMENT = 4

    work = models.ForeignKey("meleager.Work", on_delete=models.CASCADE, related_name="editions")
    edition_type = models.IntegerField(choices=EditionType.choices)
    metadata = models.JSONField("Metadata for Edition", null=False, default=dict)

    def __str__(self):
        if not self.descriptions.count():
            return f"Edition (PK {self.pk})"

        return f"{self.descriptions.first().description} ({self.descriptions.first().language})"
