from django.conf import settings
from django.db import models
from django.utils import timezone


class EditableResourceMixin(models.Model):
    """
    A resource that holds metadata on its edition.
    """

    created_at = models.DateTimeField(default=timezone.now)
    updated_at = models.DateTimeField(default=timezone.now)

    creator = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.PROTECT,
        related_name="%(app_label)s_%(class)s_creator",
        related_query_name="%(app_label)s_%(class)ss_creator",
        help_text="The creator of this resource.",
        null=True,
    )

    last_editor = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.PROTECT,
        related_name="%(app_label)s_%(class)s_last_editor",
        related_query_name="%(app_label)s_%(class)ss_last_editor",
        help_text="The last editor of this resource.",
        null=True,
    )

    class Meta:
        abstract = True
