from django.db import models


class AlternativeURNResourceMixin(models.Model):
    """
    A resource can have a main URN and a set of alternatives URNs.
    """

    urn = models.ForeignKey(
        "meleager.URN",
        on_delete=models.PROTECT,
        blank=True,
        null=True,
        related_name="%(app_label)s_%(class)s_urn",
        related_query_name="%(app_label)s_%(class)ss_urn",
        help_text="The URN of this resource.",
    )

    alternative_urns = models.ManyToManyField(
        "meleager.URN",
        related_name="%(app_label)s_%(class)s_alternative_urns",
        related_query_name="%(app_label)s_%(class)ss_alternative_urns",
        help_text="The alternative URNs of this resource.",
        blank=True,
    )

    class Meta:
        abstract = True
