from django.db import models
from django.urls import reverse

from .mixins import (AlternativeURNResourceMixin, DescriptableResourceMixin,
                     EditableResourceMixin, ValidableResourceMixin)


class Scholium(
    EditableResourceMixin,
    ValidableResourceMixin,
    DescriptableResourceMixin,
    AlternativeURNResourceMixin,
    models.Model,
):
    number = models.IntegerField("Scholium ID", db_index=True, null=True)

    passage = models.ForeignKey(
        "meleager.Passage",
        related_name="scholia",
        on_delete=models.CASCADE,
        null=True,
    )
    city = models.ForeignKey(
        "meleager.City",
        related_name="scholia",
        on_delete=models.SET_NULL,
        null=True, blank=True,
    )

    manuscripts = models.ManyToManyField(
        "meleager.Manuscript", help_text="The image of the manuscript for this passage.", blank=True,
    )

    images = models.ManyToManyField(
        "meleager.Medium",
        related_name="scholia",
        help_text="The images associated to this resource.", blank=True,
    )

    comments = models.ManyToManyField(
        "meleager.Comment",
        related_name="scholia",
        help_text="The comments of this resource.", blank=True,
    )

    keywords = models.ManyToManyField(
        "meleager.Keyword",
        related_name="scholia",
        help_text="The keywords that tag this resource.", blank=True,
    )

    def get_absolute_url(self):
        return reverse(
            "web:scholium",
            self.passage.book.number,
            self.passage.fragment,
            self.passage.sub_fragment,
        )
    
    def __str__(self):
        return f"Scholium {self.passage.book.number}.{self.passage.fragment}{self.passage.sub_fragment}.{self.number}"

    class Meta:
        verbose_name_plural = "Scholia"
