from django.test import TestCase
from django.urls import reverse

from meleager.models import Book, Comment, Description, Passage


class CommentsForms(TestCase):
    fixtures = ["test_data.json"]

    @classmethod
    def setUpTestData(cls):
        book = Book.objects.first()
        cls.passage = Passage.objects.get(book=book, fragment=12)

    def test_create_comment(self):
        self.assertEquals(self.passage.comments.count(), 0)

        self.client.post(
            reverse("web:comment-create", args=(self.passage.pk,)),
            data={
                "comment_title": "comment_title",
                "description": "description",
                "language": "fra",
            },
        )

        first_description = self.passage.comments.first().descriptions.first()
        self.assertEquals(first_description.description, "description")
        self.assertEquals(first_description.language.code, "fra")

    def test_update_comment(self):
        comment = Comment.objects.create(
            comment_title="comment_title",
        )
        self.passage.comments.add(comment)

        description = Description.objects.create(
            description="old_content", language_id="eng"
        )
        comment.descriptions.add(description)
        self.assertEquals(comment.descriptions.first().description, "old_content")
        self.assertEquals(comment.descriptions.first().language.code, "eng")

        self.client.post(
            reverse(
                "web:comment-update",
                kwargs={"comment_pk": comment.pk, "passage_pk": self.passage.pk},
            ),
            data={
                "description": "new_content",
                "language": "spa",
            },
        )

        self.assertEquals(comment.descriptions.first().description, "new_content")
        self.assertEquals(comment.descriptions.first().language.code, "spa")

    def test_delete_comment(self):
        comment = Comment.objects.create(
            comment_title="comment_title",
        )
        self.passage.comments.add(comment)

        description = Description.objects.create(
            description="description", language_id="eng"
        )
        comment.descriptions.add(description)
        self.assertEquals(self.passage.comments.count(), 1)

        self.client.post(
            reverse(
                "web:comment-delete",
                kwargs={"comment_pk": comment.pk, "passage_pk": self.passage.pk},
            ),
            data={
                "pk": description.pk,
            },
        )

        self.assertEquals(self.passage.comments.count(), 0)
