from django.shortcuts import get_object_or_404, render, redirect
from django.views import generic

from meleager.models import Author, Passage
from web.forms.author import AuthorForm
from web.forms.passage import AddAuthorToPassageForm, RemoveFromPassageForm


class AuthorCreate(generic.CreateView):
    model = Author
    form_class = AuthorForm
    # fields = "__all__"
    template_name = "web/author/form.html"


class AuthorUpdate(generic.UpdateView):
    model = Author
    form_class = AuthorForm
    template_name = "web/author/form.html"


class AuthorDetail(generic.DetailView):
    model = Author
    context_object_name = "author"
    template_name = "web/author/detail.html"


class AuthorList(generic.ListView):
    model = Author
    template_name = "web/author/list.html"
    context_object_name = "authors"


def add_author_to_passage(request, passage_pk):
    passage = get_object_or_404(Passage, pk=passage_pk)
    if request.method == "GET":
        return render(
            request,
            "web/passage/add_author_to_passage.html",
            {
                "passage": passage,
                "add_author_to_passage_form": AddAuthorToPassageForm(),
            },
        )

    add_author_to_passage_form = AddAuthorToPassageForm(request.POST)

    if add_author_to_passage_form.is_valid():
        author_pk = add_author_to_passage_form.cleaned_data.get("author")
        author = Author.objects.get(pk=author_pk)
        passage.authors.add(author)
        return redirect(f"{passage.get_absolute_url()}#authors")
    else:
        return render(
            request,
            "web/passage/add_author_to_passage.html",
            {
                "passage": passage,
                "add_author_to_passage_form": add_author_to_passage_form,
            },
        )


def remove_author_from_passage(request, passage_pk):
    passage = get_object_or_404(Passage, pk=passage_pk)
    if request.method == "GET":
        return render(
            request,
            "web/passage/remove_author_from_passage_form.html",
            {
                "passage": passage,
                "remove_author_from_passage_form": RemoveFromPassageForm(),
            },
        )

    remove_author_from_passage_form = RemoveFromPassageForm(request.POST)

    if remove_author_from_passage_form.is_valid():
        author_pk = remove_author_from_passage_form.cleaned_data.get("pk")
        author = Author.objects.get(pk=author_pk)
        passage.authors.remove(author)
        return redirect(f"{passage.get_absolute_url()}#authors")
    else:
        return render(
            request,
            "web/passage/remove_author_from_passage_form.html",
            {
                "passage": passage,
                "remove_author_from_passage_form": remove_author_from_passage_form,
            },
        )
