from django.views import generic
from django.shortcuts import get_object_or_404, redirect, render

from meleager.models import City, Passage
from web.forms.city import CityForm
from web.forms.passage import AddCityToPassageForm, RemoveFromPassageForm


def add_city_to_passage(request, passage_pk):
    passage = get_object_or_404(Passage, pk=passage_pk)
    if request.method == "GET":
        return render(
            request,
            "web/passage/add_city_to_passage.html",
            {
                "passage": passage,
                "add_city_to_passage_form": AddCityToPassageForm(),
            },
        )

    add_city_to_passage_form = AddCityToPassageForm(request.POST)

    if add_city_to_passage_form.is_valid():
        city_pk = add_city_to_passage_form.cleaned_data.get("city")
        city = City.objects.get(pk=city_pk)
        passage.cities.add(city)
        return redirect(f"{passage.get_absolute_url()}#cities")
    else:
        return render(
            request,
            "web/passage/add_city_to_passage.html",
            {
                "passage": passage,
                "add_city_to_passage_form": add_city_to_passage_form,
            },
        )


def remove_city_from_passage(request, passage_pk):
    passage = get_object_or_404(Passage, pk=passage_pk)
    if request.method == "GET":
        return render(
            request,
            "web/passage/remove_city_from_passage.html",
            {
                "passage": passage,
                "remove_city_from_passage_form": RemoveFromPassageForm(),
            },
        )

    remove_city_from_passage_form = RemoveFromPassageForm(request.POST)

    if remove_city_from_passage_form.is_valid():
        city_pk = remove_city_from_passage_form.cleaned_data.get("pk")
        city = City.objects.get(pk=city_pk)
        passage.cities.remove(city)
        return redirect(f"{passage.get_absolute_url()}#cities")
    else:
        return render(
            request,
            "web/passage/remove_city_from_passage.html",
            {
                "passage": passage,
                "remove_city_from_passage_form": remove_city_from_passage_form,
            },
        )


class CityList(generic.ListView):
    model = City
    template_name = "web/city/list.html"
    context_object_name = "cities"


class CityDetail(generic.DetailView):
    model = City
    template_name = "web/city/detail.html"
    context_object_name = "city"


class CityCreate(generic.CreateView):
    model = City
    template_name = "web/city/form.html"
    form_class = CityForm


class CityUpdate(generic.UpdateView):
    model = City
    template_name = "web/city/form.html"
    form_class = CityForm
