from django.shortcuts import get_object_or_404, redirect, render
from django.views import generic

from meleager.models import Comment, Description, Passage
from web.forms.comment import CommentForm
from web.forms.comment_description import (
    CommentCreateForm,
    CommentDeleteForm,
    CommentUpdateForm,
)
from web.forms.description import DescriptionForm


def comment_create(request, passage_pk):
    passage = get_object_or_404(Passage, pk=passage_pk)
    if request.method == "GET":
        return render(
            request,
            "web/passage/comment_create.html",
            {
                "passage": passage,
                "comment_create_form": CommentCreateForm(),
            },
        )
    description_form = DescriptionForm(request.POST)
    comment_form = CommentForm(request.POST)
    comment_create_form = CommentCreateForm(request.POST)

    if description_form.is_valid() and comment_form.is_valid():
        description = description_form.save()
        comment = comment_form.save()
        comment.descriptions.add(description)
        passage.comments.add(comment)
        return redirect(f"{passage.get_absolute_url()}#comment-{comment.pk}")
    else:
        return render(
            request,
            "web/comment_create.html",
            {
                "passage": passage,
                "comment_create_form": comment_create_form,
            },
        )


def comment_delete(request, passage_pk, comment_pk):
    passage = get_object_or_404(Passage, pk=passage_pk)
    if request.method == "GET":
        return render(
            request,
            "web/passage/comment_delete.html",
            {
                "passage": passage,
                "comment_delete": CommentDeleteForm(),
            },
        )

    comment = get_object_or_404(Comment, pk=comment_pk)
    comment_delete_form = CommentDeleteForm(request.POST)

    if comment_delete_form.is_valid():
        description_pk = comment_delete_form.cleaned_data.get("pk")
        description = Description.objects.get(pk=description_pk)
        description.delete()
        comment.delete()
        return redirect(f"{passage.get_absolute_url()}#comments")
    else:
        return render(
            request,
            "web/passage/comment_delete.html",
            {
                "passage": passage,
                "comment_delete_form": comment_delete_form,
            },
        )


def comment_update(request, passage_pk, comment_pk):
    passage = get_object_or_404(Passage, pk=passage_pk)
    if request.method == "GET":
        return render(
            request,
            "web/passage/comment_update.html",
            {
                "passage": passage,
                "comment_update_form": CommentUpdateForm(),
            },
        )

    comment_update_form = CommentUpdateForm(request.POST)
    passage = get_object_or_404(Passage, pk=passage_pk)
    description_form = DescriptionForm(request.POST)
    if description_form.is_valid():
        comment_obj = get_object_or_404(Comment, pk=comment_pk)
        # TODO: handle multiple descriptions?
        description_obj = comment_obj.descriptions.first()
        description_obj.description = description_form.cleaned_data["description"]
        description_obj.language = description_form.cleaned_data["language"]
        description_obj.save()
        return redirect(f"{passage.get_absolute_url()}#comment-{comment_pk}")
    else:
        return render(
            request,
            "web/comment_update.html",
            {
                "passage": passage,
                "comment_update_form": comment_update_form,
            },
        )


class CommentDetail(generic.DetailView):
    model = Comment
    template_name = "web/comment/detail.html"
