from django.shortcuts import get_object_or_404, redirect, render

from meleager.models import Description, Passage
from web.forms.passage_description import (
    DescriptionCreateForm,
    DescriptionDeleteForm,
    DescriptionUpdateForm,
)
from web.forms.description import DescriptionForm


def description_create(request, passage_pk):
    passage = get_object_or_404(Passage, pk=passage_pk)
    if request.method == "GET":
        return render(
            request,
            "web/passage/description_create.html",
            {
                "passage": passage,
                "description_create_form": DescriptionCreateForm(),
            },
        )

    description_create_form = DescriptionForm(request.POST)

    if description_create_form.is_valid():
        description = description_create_form.save()
        passage.descriptions.add(description)
        return redirect(f"{passage.get_absolute_url()}#description-{description.pk}")
    else:
        return render(
            request,
            "web/description_create.html",
            {
                "passage": passage,
                "description_create_form": description_create_form,
            },
        )


def description_delete(request, passage_pk, description_pk):
    passage = get_object_or_404(Passage, pk=passage_pk)
    if request.method == "GET":
        return render(
            request,
            "web/passage/description_delete.html",
            {
                "passage": passage,
                "description_delete": DescriptionDeleteForm(),
            },
        )

    description_delete_form = DescriptionDeleteForm(request.POST)

    if description_delete_form.is_valid():
        description_pk = description_delete_form.cleaned_data.get("pk")
        description = Description.objects.get(pk=description_pk)
        description.delete()
        return redirect(f"{passage.get_absolute_url()}#descriptions")
    else:
        return render(
            request,
            "web/passage/description_delete.html",
            {
                "passage": passage,
                "description_delete_form": description_delete_form,
            },
        )


def description_update(request, passage_pk, description_pk):
    passage = get_object_or_404(Passage, pk=passage_pk)
    if request.method == "GET":
        return render(
            request,
            "web/passage/description_update.html",
            {
                "passage": passage,
                "description_update_form": DescriptionUpdateForm(),
            },
        )

    description_update_form = DescriptionUpdateForm(request.POST)
    passage = get_object_or_404(Passage, pk=passage_pk)
    description_form = DescriptionForm(request.POST)
    if description_form.is_valid():
        description = Description.objects.get(pk=description_pk)
        description.description = description_form.cleaned_data["description"]
        description.language = description_form.cleaned_data["language"]
        description.save()
        return redirect(f"{passage.get_absolute_url()}#description-{description.pk}")
    else:
        return render(
            request,
            "web/description_update.html",
            {
                "passage": passage,
                "description_update_form": description_update_form,
            },
        )
