from django.shortcuts import get_object_or_404, render, redirect

from meleager.models import Keyword, Passage
from web.forms.passage import AddKeywordToPassageForm, RemoveFromPassageForm


def keywords(request):
    keywords = Keyword.objects.all()
    return render(request, "web/keywords.html", {"keywords": keywords})


def add_keyword_to_passage(request, passage_pk):
    passage = get_object_or_404(Passage, pk=passage_pk)
    if request.method == "GET":
        return render(
            request,
            "web/passage/add_keyword_to_passage.html",
            {
                "passage": passage,
                "add_keyword_to_passage_form": AddKeywordToPassageForm(),
            },
        )

    add_keyword_to_passage_form = AddKeywordToPassageForm(request.POST)

    if add_keyword_to_passage_form.is_valid():
        keyword_pk = add_keyword_to_passage_form.cleaned_data.get("keyword")
        keyword = Keyword.objects.get(pk=keyword_pk)
        passage.keywords.add(keyword)
        return redirect(f"{passage.get_absolute_url()}#keywords")
    else:
        return render(
            request,
            "web/passage/add_keyword_to_passage.html",
            {
                "passage": passage,
                "add_keyword_to_passage_form": add_keyword_to_passage_form,
            },
        )


def remove_keyword_from_passage(request, passage_pk):
    passage = get_object_or_404(Passage, pk=passage_pk)
    if request.method == "GET":
        return render(
            request,
            "web/passage/remove_keyword_from_passage.html",
            {
                "passage": passage,
                "remove_keyword_from_passage_form": RemoveFromPassageForm(),
            },
        )

    remove_keyword_from_passage_form = RemoveFromPassageForm(request.POST)

    if remove_keyword_from_passage_form.is_valid():
        keyword_pk = remove_keyword_from_passage_form.cleaned_data.get("pk")
        keyword = Keyword.objects.get(pk=keyword_pk)
        passage.keywords.remove(keyword)
        return redirect(f"{passage.get_absolute_url()}#keywords")
    else:
        return render(
            request,
            "web/passage/remove_keyword_from_passage.html",
            {
                "passage": passage,
                "remove_keyword_from_passage_form": remove_keyword_from_passage_form,
            },
        )
